# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-05-02 23:11
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('examples', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='userinfo',
            name='avg1',
            field=models.FloatField(blank=True, null=True, verbose_name='Average 1'),
        ),
    ]
