# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-04-26 22:59
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='UserInfo',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('age', models.IntegerField(verbose_name='Age')),
                ('sex', models.CharField(choices=[(0, 'M'), (1, 'F')], max_length=1, verbose_name='Sex')),
            ],
        ),
    ]
